/* ************************************************************************** *
 *  Filename: control_com.c
 *  Functions: control_com_kick(), control_com_dash(), control_com_turn().
 *
 *  This client program is developed based on a sample client.
 *  Client Program (Control Command Routines).
 *  by Neo Say Poh/ Eldwin Tan Say Lin.
 * ************************************************************************** */

#include "Agent.h"

/* ************************************************************************** *
 * control_com_kick() - Send a 'KICK' command to server.
 *
 * Args:
 *		sock : 	socket
 *		pow  : 	power
 *		dir  : 	direction
 * Return type:
 * 		Bool : 	TRUE  - successful
 *	       	   	FALSE - Failed
 * ************************************************************************** */
Bool control_com_kick(Socket *sock, int pow, double dir)
{
    	char 	mesg_buf[BUFSIZE];

    	/* check for out of range */	
    	if (pow<0)
		pow=0;
    	else if (pow>100)
		pow=100;
    	if (dir<-180.0)
		dir+=360.0;
    	else if (dir>180.0)
		dir-=360.0;

    	sprintf(mesg_buf,"(kick %d %d)\n",pow,(int)dir);
    	if (!send_message(*sock, mesg_buf))
		return FALSE ;
    	else
		return TRUE;
}

/* ************************************************************************** *
 * control_com_dash() - Send a 'DASH' command to server.
 * 
 * Args:
 *		sock : 	socket
 * 		pow  : 	power
 * Return type:
 *		Bool : 	TRUE  - successful
 *		       	FALSE - Failed
 * ************************************************************************** */
Bool control_com_dash(Socket *sock, int pow)
{
    	char 	mesg_buf[BUFSIZE];
    
    	/* check for out of range */
    	if (pow>100)
		pow=100;
    	else if (pow<-30)
		pow=-30;

    	sprintf(mesg_buf,"(dash %d)\n",pow);
    	if (!send_message(*sock, mesg_buf))
		return FALSE ;
    	else
		return TRUE ;
}

/* ************************************************************************** *
 * control_com_turn() - Send a 'TURN' command to server.
 *
 * Args:
 * 		sock   : socket
 *		moment : angle
 * Return type:
 *		Bool : 	TRUE  - successful
 *	       	   	FALSE - Failed
 * ************************************************************************** */
Bool control_com_turn(Socket *sock, double moment)
{
    	char 	mesg_buf[BUFSIZE];
	
    	/* check for out of range */
    	if (moment<-180.0)
		moment+=360.0;
    	else if (moment>180.0)
		moment-=360.0;

    	sprintf(mesg_buf,"(turn %d)\n",(int)moment);
    	if (!send_message(*sock, mesg_buf))
		return FALSE ;
    	else
		return TRUE ;
}
