/* ************************************************************************** *
 *  Filename: misc.c
 *  Functions: get_angle(), get_my_posi(), get_para(), get_player_para(),
 *			   init_condition().
 *
 *  This client program is developed based on a sample client.
 *  Client Program (Miscellance Routines).
 *  by Neo Say Poh/ Eldwin Tan Say Lin.
 * ************************************************************************** */

#include "Agent.h"

double mx,my,mdir; /* player's position: (mx,my) */
double bx,by,blocation[2];
int    player;
char   side;
int    time_counter=0;

/* Position of flags, goals and players. */
double flag_posi[53][2]={{CENTER,CENTER},{CENTER,-FIELD_WIDTH},
			 {-(FIELD_LENGTH+5),CENTER},{-FIELD_LENGTH,-FIELD_WIDTH},
			 {-FIELD_LENGTH,FIELD_WIDTH},
			 {-(FIELD_LENGTH-PENALTY_AREA_LENGTH),CENTER},	
			 {-(FIELD_LENGTH-PENALTY_AREA_LENGTH),-PENALTY_AREA_WIDTH},
			 {-(FIELD_LENGTH-PENALTY_AREA_LENGTH),PENALTY_AREA_WIDTH},
			 {-FIELD_LENGTH,-(GOAL_WIDTH/2)},{-FIELD_LENGTH,(GOAL_WIDTH/2)},
			 {-(FIELD_LENGTH+5),-10},{-(FIELD_LENGTH+5),10},{-(FIELD_LENGTH+5),-20},
			 {-(FIELD_LENGTH+5),20},{-(FIELD_LENGTH+5),-30},{-(FIELD_LENGTH+5),30},
			 {-10,-(FIELD_WIDTH+5)},{-20,-(FIELD_WIDTH+5)},{-30,-(FIELD_WIDTH+5)},
			 {-40,-(FIELD_WIDTH+5)},{-50,-(FIELD_WIDTH+5)},
			 {-10,FIELD_WIDTH+5},{-20,FIELD_WIDTH+5},{-30,FIELD_WIDTH+5},
		         {-40,FIELD_WIDTH+5},{-50,FIELD_WIDTH+5},
			 {CENTER,-(FIELD_WIDTH+5)},{CENTER,FIELD_WIDTH},
			 {FIELD_LENGTH+5,CENTER},{FIELD_LENGTH,FIELD_WIDTH},
			 {FIELD_LENGTH,-FIELD_WIDTH},
			 {FIELD_LENGTH-PENALTY_AREA_LENGTH,CENTER},
			 {FIELD_LENGTH-PENALTY_AREA_LENGTH,PENALTY_AREA_WIDTH},
                         {FIELD_LENGTH-PENALTY_AREA_LENGTH,-PENALTY_AREA_WIDTH},
			 {FIELD_LENGTH,(GOAL_WIDTH/2)},{FIELD_LENGTH,-(GOAL_WIDTH/2)},
			 {FIELD_LENGTH+5,10},{FIELD_LENGTH+5,-10},{FIELD_LENGTH+5,20},
                         {FIELD_LENGTH+5,-20},{FIELD_LENGTH+5,30},{FIELD_LENGTH+5,-30},
			 {10,(FIELD_WIDTH+5)},{20,(FIELD_WIDTH+5)},{30,(FIELD_WIDTH+5)},
                         {40,(FIELD_WIDTH+5)},{50,(FIELD_WIDTH+5)},
			 {10,-(FIELD_WIDTH+5)},{20,-(FIELD_WIDTH+5)},{30,-(FIELD_WIDTH+5)},
                         {40,-(FIELD_WIDTH+5)},{50,-(FIELD_WIDTH+5)},{CENTER,FIELD_WIDTH+5}};
double goal_posi[2][2]={{-FIELD_LENGTH,CENTER}, {FIELD_LENGTH,CENTER}};
int  init_posi[11][10]={{-51,0},{-30,0},{-25,-20},{-25,20},{-20,0},{-10,-10},
                        {-10,10},{-5,-23},{-5,23},{-1,0},{-1,3}};


struct relative_address flag[FLAG_COUNT],goal[GOAL_COUNT],line[LINE_COUNT];
struct sequence_list s_list;

/* ************************************************************************** *
 * get_angle() - Calculate the angle from my position to (dx,dy).
 * 
 * Args:
 * 		dx : x-axis for calculation of angle.
 *		dy : y-axis for calculation of angle.
 * Return type:
 *		double : angle after calculations.
 * ************************************************************************** */
double get_angle(double dx, double dy)
{
	double dist=sqrt(dx*dx+dy*dy);
    	double a,b,dir;

    	dx/=dist;
	dy/=dist;
    	a=asin(dy);
    	b=acos(dx);
    	dir=b*180.0/PI;
    	if (a<0)
		dir=360-dir;
	return dir;
}
	

/* ************************************************************************** *
 * get_my_posi() - get my position from the sight information of line(s).
 *
 * Arg:
 *		void.
 * Return type:
 *		Bool : 	TRUE  - successful.
 *	       	   	FALSE - Failed.
 * ************************************************************************** */
Bool get_my_posi(void)
{
    	int  	lnum=0,ls[2],phase;		  /* lnum - stores index of line */
    	int  	i,j,cond_flag=0;
    	double 	dist,px,py,lx,ly,dir,ldir; /* dir - dir to goal or flag, ldir - dir to line */

	for (i=0;i<LINE_COUNT;i++)
		if (line[i].count==time_counter)
		    	ls[lnum++]=i;
    	if (lnum==2)
		if (line[ls[0]].d[0]>line[ls[1]].d[0])
			phase=0;
		else
		    	phase=1;
    	if (lnum==0) /* out of the field and no lines in sight */
		return FALSE;

	for (j=0;j<lnum;j++)
    	{
		i=ls[j];
		dist=line[i].d[0]*cos(line[i].d[1]*PI/180.0);
		switch(i)
		{
	  		case 0: /* left end */
		    		if ((lnum==1) || (phase==j))
		    		{
					lx=-FIELD_LENGTH+dist;
					ldir=line[i].d[1]+180.0;
		    		} 
		    		else
		    		{
					lx=-FIELD_LENGTH-dist;
					ldir=line[i].d[1]+0.0;
		    		}
		    		cond_flag|=0x01;
		    		break;
	  		case 1: /* right end */
				if ((lnum==1) || (phase==j))
		    		{
					lx=FIELD_LENGTH-dist;
					ldir=line[i].d[1]+0.0;
				}
		    		else
		    		{
					lx=FIELD_LENGTH+dist;
					ldir=line[i].d[1]+180.0;
		    		}
				cond_flag|=0x01;
		    		break;
	  		case 2: /* top end */
		    		if ((lnum==1) || (phase==j))
				{
					ly=-FIELD_WIDTH+dist;
					ldir=line[i].d[1]+270.0;
		    		} 
		    		else
		    		{
					ly=-FIELD_WIDTH-dist;
					ldir=line[i].d[1]+90.0;
		    		}
		    		cond_flag|=0x02;
				break;
	  		case 3: /* bottom end */
		    		if ((lnum==1) || (phase==j))
		    		{
					ly=FIELD_WIDTH-dist;
					ldir=line[i].d[1]+90.0;
				}
		    		else
		    		{
					ly=FIELD_WIDTH+dist;
					ldir=line[i].d[1]+270.0;
		    		}
		    		cond_flag|=0x02;
		    		break;
		}
    	}
	if (cond_flag!=0x03)
    	{
		dist=180.0;

		/* get the nearest object's (flag/goal) position */
		for (i=0;i<FLAG_COUNT;i++)
		{
			if ((flag[i].count==time_counter) && (dist>flag[i].d[0]))
		    	{
				dist=flag[i].d[0];
				dir=flag[i].d[1];
				px=flag_posi[i][0];
				py=flag_posi[i][1];
		    	}
		}
		for (i=0;i<GOAL_COUNT;i++)
		{
			if ((goal[i].count==time_counter) && (dist>goal[i].d[0]))
		    	{
				dist=goal[i].d[0];
				dir=goal[i].d[1];
				px=goal_posi[i][0];
				py=goal_posi[i][1];
		    	}
		}

		if (cond_flag==0x01)
		{
		    	mx=lx;
		    	if (dist!=180.0)
				my=py-dist*sin((ldir+dir)*PI/180.0);
		} 
		else
		{
		    	my=ly;
		    	if (dist!=180.0)
				mx=px-dist*cos((ldir+dir)*PI/180.0);
		}
    	} 
	else
    	{
		mx=lx;
		my=ly;
    	}
    	mdir=ldir;
	
    	return TRUE;
}
	

/* ************************************************************************** *
 * get_para() - filling in an object's relative address structure with
 *				parameters: dist dir dist_chg dir_chg.
 *
 * Args:
 *		st  : relative address structure of objects (ball, player, goal,
 *			  flag, line).
 *		buf : message buffer (consisting of: dist dir dist_chg dir_chg).
 * Return type:
 *		char * : points to the next ObjInfo.
 * ************************************************************************** */
char *get_para(struct relative_address *st,char *buf)
{
    	double 	d; /* temp. storage for Object's parameters */
    	int  	i=0,m_flag; /* i - index, m_flag - test for negative value */

    	st->count=time_counter;
    
    	while(1)
    	{
		d=0.0; m_flag=0;
		while((*buf!=')') && (*buf!=' '))
		{
		    	if (*buf=='.') /* decimal value */
		    	{
				buf++;
				d+=(double)(*buf-'0')/10.0;
		    	} 
		    	else if (*buf=='-') /* negative value */
				m_flag=1;
		    	else
				d=d*10.0+(double)(*buf-'0');
		    	buf++;
		}
		if (d==180.0)
			d=0.0;
		if (m_flag)
		    	d=-d;
		st->d[i++]=d;
		if (*buf==')')
		{
		    	while (i<4)
				st->d[i++]=-360.0;
		    	st->dist=st->d[0];
		    	st->dir=(int)st->d[1];
			st->vdir=0;
			return (buf+1);
		}
		else
			buf++;
	}
}


/* ************************************************************************** *
 * get_player_para() - filling in player's relative address structure with
 *		       parameters: dist dir dist_chg dir_chg face_dir.
 *
 * Args:
 *		st  : player's relative address structure.
 *		buf : message buffer (consisting of: dist dir dist_chg dir_chg).
 * Return type:
 *		char * : points to the next ObjInfo.
 * ************************************************************************** */
char *get_player_para(struct player_relative_address *st,char *buf)
{
	double 	d;		 /* temp. storage for Object's parameters */
	int  	i=0,m_flag; /* i - index, m_flag - test for negative value */

	st->count=time_counter;

	while(1)
	{
		d=0.0; m_flag=0;
		while((*buf!=')') && (*buf!=' '))
		{
			if (*buf=='.') /* decimal value */
			{
				buf++;
				d+=(double)(*buf-'0')/10.0;
			}
			else if (*buf=='-') /* negative value */
				m_flag=1;
			else
				d=d*10.0+(double)(*buf-'0');
			buf++;
		}
		if (d==180.0)
			d=0.0;
		if (m_flag)
			d=-d;
		st->d[i++]=d;
		if (*buf==')')
		{
			while (i<5)
				st->d[i++]=-360.0;
			st->dist=st->d[0];
			st->dir=(int)st->d[1];
			st->vdir=0;
			return (buf+1);
		}
		else
			buf++;
	}
}


/* ************************************************************************** *
 * init_condition() - opens a connection with the soccer server, moves the
 *		      player to the correct position and change view.
 *
 * Args:
 *		sock   		: socket.
 *		t_name 		: team name.
 *		goalie_flag : 	TRUE  - player is goal-keeper.
 *			  	FALSE - not goal-keeper.
 * Return type:
 *		Bool : 	TRUE  - successful.
 *	       	   	FALSE - Failed.
 * ************************************************************************** */
Bool init_condition(Socket *sock, char *t_name, int goalie_flag)
{	
    	char 	buf[MAXMESG],playmodestr[BUFSIZE];
    	char 	init_mesg[30],mov_mesg[30];
    	char 	serv_response[16];	/* server's response to init command */
    	int 	i;					/* for searching purpose only */
	char 	sensor_info[50];

	if (goalie_flag)
       	 	sprintf(init_mesg,"(init %s (version 4.00) (goalie))\n",t_name);
    	else
       	 	sprintf(init_mesg,"(init %s (version 4.00))\n",t_name);
	
	if (!send_message(*sock, init_mesg))
		return FALSE;

    	/* Wait for server's reply */
    	while (receive_message(sock, buf, MAXMESG)<=0);

    	sscanf(buf, "(%s ", serv_response);
    	if (!strcmp(serv_response,INIT_OK))
    	{
		sscanf(buf, "(init %c %d %s)", &side, &player, playmodestr);

		player-=1;	/* player index starting from 0 */

		/* Search & replace ')' with '\0' */
		for (i=0; playmodestr[i]!=')'; i++);
		playmodestr[i] = '\0';

		if (strcmp(BEFORE_KICK_OFF,playmodestr)!=0)
    		{
       			    printf("match is on play\n");
	    		return FALSE;
        	}
     
		mx=(double)init_posi[player][0];
       		my=(double)init_posi[player][1];
       		s_list.count=0;

		do 
		{
			if (side=='l')
				sprintf(mov_mesg,"(move %d %d)\n", init_posi[player][0], init_posi[player][1]);
			else
			{
				if (player==9)
				{
					mx=0.0;
					my=-11.0;
					sprintf(mov_mesg,"(move 0 -11)\n");
				}
				else if (player==10)
				{
					mx=0.0;
					my=11.0;
					sprintf(mov_mesg,"(move 0 11)\n");
				}
				else
					sprintf(mov_mesg,"(move %d %d)\n", init_posi[player][0], init_posi[player][1]);
			}
        		usleep(100000);

        		if (!send_message(*sock, mov_mesg))
				return FALSE;
        		usleep(100000);

			while (receive_message(sock, buf, MAXMESG)<=0);
        		sscanf(buf, "(%s ", sensor_info);
        		if (!strcmp(sensor_info, SEE))
            			see(buf,t_name);
        		else
            			continue;

        	} while (!(get_my_posi())); /* move to init_posi until successful */

		if (!send_message(*sock, "(change_view narrow high)\n"))
			return FALSE;
        	bx=0; by=0;

        	while(1) /* wait for referee to start match */
        	{
 	    		while (receive_message(sock, buf, MAXMESG)<=0);

			sscanf(buf, "(%s ", serv_response);

	    		if (!strcmp(serv_response, HEAR))
	    		{
	       		 	if (hear(buf, t_name)) /* hear referee's msg: kick_off_? */
		    			return TRUE;
	    		}
       	 	}
    	}
    	else
    	{
       	 	printf("initialize error\n");
		return FALSE;
    	}
}
